@@ -59,6 +59,7 @@ def process_args(sysargs=None):
         "resume": do_resume,
         "pause": do_pause,
         "spider": do_spider,
+        "rawquery": do_rawquery,
     }
 
     parser = DefaultHelpArgParser(description="Execute a workflow")
@@ -117,6 +118,16 @@ def process_args(sysargs=None):
         subparsers.add_parser("query", help="Search known workflows by some criteria")
     )
 
+    add_cleanup_args(
+        subparsers.add_parser("cleanup", help="Cleanup the central db of workflow runs")
+    )
+
+    add_rawquery_args(
+        subparsers.add_parser(
+            "rawquery", help="Perform a raw SQL query on the sqlite database of a task"
+        )
+    )
+
     subparsers.add_parser("version", help="Print the versions of Janis and exit")
     subparsers.add_parser(
         "docs", help="Attempts to open Janis documentation using `webbrowser`"
@@ -192,6 +203,12 @@ def add_spider_args(parser):
 
 def add_resume_args(parser):
     parser.add_argument("wid", help="WID to resume")
+    parser.add_argument(
+        "--foreground",
+        action="store_true",
+        help="By default, the workflow will be resubmitted per your template's recommendation. "
+        "(This is often in the background). Use this option to force running in the foreground.",
+    )
     return parser
 
 
@@ -250,51 +267,71 @@ def add_translate_args(parser):
 
 
 def add_inputs_args(parser):
+    from janis_core import HINTS, HintEnum
+
     parser.add_argument("workflow", help="workflow to generate inputs for")
     parser.add_argument("-c", "--config", help="Path to config file")
 
     parser.add_argument("-o", "--output", help="file to output to, else stdout")
+
     parser.add_argument(
+        "--resources",
+        action="store_true",
+        help="Add resource overrides into inputs file (eg: runtime_cpu / runtime_memory)",
+    )
+
+    parser.add_argument(
+        "--json", action="store_true", help="Output to JSON instead of yaml"
+    )
+    parser.add_argument("-i", "--inputs", help="additional inputs to pull values from")
+
+    input_manip_args = parser.add_argument_group("filtering")
+
+    input_manip_args.add_argument(
+        "-r",
+        "--recipes",
+        help="If you have values available from these recipes, they will be ignored from the output",
+        nargs="+",
+    )
+
+    input_manip_args.add_argument(
         "-a",
         "--all",
         action="store_true",
         help="Include inputs that have default values",
     )
 
-    parser.add_argument(
+    input_manip_args.add_argument(
         "--user",
         action="store_true",
         help="Only include user inputs (ignore inputs with a qualityType of static or configuration). "
         "Note, this is only available for pilelines that have inputs that are annotated with input qualities.",
     )
-    parser.add_argument(
+    input_manip_args.add_argument(
         "--static",
         action="store_true",
         help="Include all inputs with a quality type of static or configuration. "
         "Note, this is only available for pilelines that have inputs that are annotated with input qualities.",
     )
 
-    parser.add_argument(
-        "-r",
-        "--recipes",
-        help="If you have values available from these recipes, they will be ignored from the output",
-        nargs="+",
-    )
-    parser.add_argument(
-        "--resources",
-        action="store_true",
-        help="Add resource overrides into inputs file (eg: runtime_cpu / runtime_memory)",
-    )
-    parser.add_argument(
-        "--json", action="store_true", help="Output to JSON instead of yaml"
-    )
-    parser.add_argument("-i", "--inputs", help="additional inputs to pull values from")
-    parser.add_argument(
+    # add hints
+    hint_args = parser.add_argument_group("hints")
+    for HintType in HINTS:
+        if issubclass(HintType, HintEnum):
+            hint_args.add_argument(
+                "--hint-" + HintType.key(), choices=HintType.symbols()
+            )
+
+    # workflow selection
+
+    selection = parser.add_argument_group("selecting workflow")
+
+    selection.add_argument(
         "-n",
         "--name",
         help="If you have multiple workflows in your file, help Janis out to select the right workflow to run",
     )
-    parser.add_argument(
+    selection.add_argument(
         "--no-cache",
         help="Force re-download of workflow if remote",
         action="store_true",
@@ -366,7 +403,7 @@ def add_run_args(parser):
     parser.add_argument(
         "--development",
         action="store_true",
-        help="Apply common settings (--keep-execution-dir + --mysql) to support incremental development of a pipeline",
+        help="Apply common settings (--keep-execution-dir) to support incremental development of a pipeline",
     )
 
     # input manipulation
@@ -485,7 +522,12 @@ def add_run_args(parser):
     beta_args.add_argument(
         "--mysql",
         action="store_true",
-        help="BETA: Run MySQL for persistence with Cromwell",
+        help="BETA: Ask Janis to run a managed MySQL for persistence with Cromwell",
+    )
+    beta_args.add_argument(
+        "--no-database",
+        action="store_true",
+        help="Skip running a database with Cromwell",
     )
 
     beta_args.add_argument(
@@ -513,6 +555,13 @@ def add_query_args(parser):
     return parser
 
 
+def add_rawquery_args(parser):
+    parser.add_argument("wid", help="Workflow ID or task directory")
+    parser.add_argument("query", help="RAW SQL query")
+
+    return parser
+
+
 def check_logger_args(args):
     level = LogLevel.INFO
     if args.verbose:
@@ -599,12 +648,12 @@ def do_docs(args):
 
 def do_watch(args):
     wid = args.wid
-    tm = ConfigManager.manager().from_wid(wid)
+    tm = ConfigManager.manager().from_wid(wid, readonly=True)
     tm.watch()
 
 
 def do_resume(args):
-    resume(args.wid)
+    resume(args.wid, foreground=args.foreground)
 
 
 def do_pause(args):
@@ -643,7 +692,7 @@ def do_rm(args):
 
 
 def do_run(args):
-    JanisConfiguration.initial_configuration(args.config)
+    jc = JanisConfiguration.initial_configuration(args.config)
 
     validation_reqs, batchrun_reqs = None, None
 
@@ -684,11 +733,18 @@ def do_run(args):
         ins = required_inputs.pop("i")
         inputs.extend(ins if isinstance(ins, list) else [ins])
 
-    mysql = args.mysql
     keep_intermediate_files = args.keep_intermediate_files is True
 
+    db_config = jc.cromwell.get_database_config_helper()
+
+    if args.mysql:
+        db_config.should_manage_mysql = True
+
+    if args.no_database:
+        db_config.skip_database = True
+
     if args.development:
-        mysql = True
+        # no change for using mysql, as a database is the default
         keep_intermediate_files = True
         JanisConfiguration.manager().cromwell.call_caching_enabled = True
 
@@ -713,7 +769,7 @@ def do_run(args):
         keep_intermediate_files=keep_intermediate_files,
         run_in_background=(args.background is True),
         run_in_foreground=(args.foreground is True),
-        mysql=mysql,
+        dbconfig=db_config,
         only_toolbox=args.toolbox,
         no_store=args.no_store,
         allow_empty_container=args.allow_empty_container,
@@ -746,6 +802,12 @@ def do_inputs(args):
     elif args.static:
         quality_type = [InputQualityType.static, InputQualityType.configuration]
 
+    hints = {
+        k[5:]: v
+        for k, v in vars(args).items()
+        if k.startswith("hint_") and v is not None
+    }
+
     outd = generate_inputs(
         args.workflow,
         all=args.all,
@@ -755,6 +817,7 @@ def do_inputs(args):
         with_resources=args.resources,
         quality_type=quality_type,
         recipes=args.recipes,
+        hints=hints,
     )
 
     if args.json:
@@ -805,6 +868,13 @@ def do_query(args):
     )
 
 
+def do_rawquery(args):
+    wid = args.wid
+    wm = ConfigManager.manager().from_wid(wid, readonly=True)
+    result = wm.database.cursor.execute(args.query).fetchall()
+    return print(tabulate.tabulate(result))
+
+
 def do_translate(args):
     translate(
         tool=args.workflow,
