@@ -32,4 +32,4 @@ class WorkflowMetadataDbKeys(Enum):
     submission_workflow = "submission_workflow"
     submission_inputs = "submission_inputs"
     submission_resources = "submission_resources"
-    should_manage_database = "should_manage_database"
+    dbconfig = "dbconfig"
