@@ -24,7 +24,16 @@ from enum import Enum
 from subprocess import call
 from typing import Optional, List, Dict, Union, Any
 
-from janis_core import InputSelector, Logger, Workflow, File, Array, LogLevel, Directory
+from janis_core import (
+    InputSelector,
+    Logger,
+    Workflow,
+    File,
+    Array,
+    LogLevel,
+    Directory,
+    Tool,
+)
 from janis_core.translations import get_translator, CwlTranslator
 from janis_core.translations.translationbase import TranslatorBase
 from janis_core.translations.wdl import apply_secondary_file_format_to_filename
@@ -40,7 +49,10 @@ from janis_assistant.engines import (
     Engine,
 )
 from janis_assistant.environments.environment import Environment
-from janis_assistant.management.configuration import JanisConfiguration
+from janis_assistant.management.configuration import (
+    JanisConfiguration,
+    JanisDatabaseConfigurationHelper,
+)
 from janis_assistant.management.filescheme import FileScheme, LocalFileScheme
 from janis_assistant.management.mysql import MySql
 from janis_assistant.management.notificationmanager import NotificationManager
@@ -88,7 +100,9 @@ class WorkflowManager:
         self.path = fully_qualify_filename(outdir)
         self.create_dir_structure(self.path)
 
-        self.database = WorkflowDbManager(wid, self.get_task_path_safe())
+        self.database = WorkflowDbManager(
+            wid, self.get_task_path_safe(), readonly=readonly
+        )
         self.environment = environment
         self.dbcontainer: MySql = None
         self.main_queue = queue.Queue()
@@ -124,7 +138,7 @@ class WorkflowManager:
     def from_janis(
         wid: str,
         outdir: str,
-        wf: Workflow,
+        tool: Tool,
         environment: Environment,
         hints: Dict[str, str],
         validation_requirements: Optional[ValidationRequirements],
@@ -136,7 +150,7 @@ class WorkflowManager:
         max_memory=None,
         keep_intermediate_files=False,
         run_in_background=True,
-        mysql=False,
+        dbconfig=None,
         allow_empty_container=False,
         check_files=True,
     ):
@@ -155,12 +169,12 @@ class WorkflowManager:
         tm.database.workflowmetadata.engine = environment.engine
         tm.database.workflowmetadata.filescheme = environment.filescheme
         tm.database.workflowmetadata.environment = environment.id()
-        tm.database.workflowmetadata.name = wf.id()
+        tm.database.workflowmetadata.name = tool.id()
         tm.database.workflowmetadata.start = DateUtil.now()
         tm.database.workflowmetadata.executiondir = None
         tm.database.workflowmetadata.keepexecutiondir = keep_intermediate_files
         tm.database.workflowmetadata.configuration = jc
-        tm.database.workflowmetadata.should_manage_database = mysql
+        tm.database.workflowmetadata.dbconfig = dbconfig
 
         # This is the only time we're allowed to skip the tm.set_status
         # This is a temporary stop gap until "notification on status" is implemented.
@@ -169,8 +183,8 @@ class WorkflowManager:
 
         spec = get_ideal_specification_for_engine(environment.engine)
         spec_translator = get_translator(spec)
-        wf_evaluate = tm.prepare_and_output_workflow_to_evaluate_if_required(
-            workflow=wf,
+        tool_evaluate = tm.prepare_and_output_workflow_to_evaluate_if_required(
+            tool=tool,
             translator=spec_translator,
             validation=validation_requirements,
             batchrun=batchrun_requirements,
@@ -187,61 +201,29 @@ class WorkflowManager:
         )
 
         tm.database.workflowmetadata.submission_workflow = os.path.join(
-            outdir_workflow, spec_translator.workflow_filename(wf_evaluate)
+            outdir_workflow, spec_translator.filename(tool_evaluate)
         )
         tm.database.workflowmetadata.submission_inputs = os.path.join(
-            outdir_workflow, spec_translator.inputs_filename(wf_evaluate)
+            outdir_workflow, spec_translator.inputs_filename(tool_evaluate)
         )
         tm.database.workflowmetadata.submission_resources = os.path.join(
-            outdir_workflow, spec_translator.dependencies_filename(wf_evaluate)
+            outdir_workflow, spec_translator.dependencies_filename(tool_evaluate)
         )
 
         tm.database.commit()
 
         if not dryrun:
-
-            # check container environment is loaded
-            tm.database.workflowmetadata.containertype = jc.container.__name__
-            tm.database.workflowmetadata.containerversion = (
-                jc.container.test_available_by_getting_version()
-            )
-
-            # this happens for all workflows no matter what type
-            tm.set_status(TaskStatus.QUEUED)
-
-            # resubmit the engine
-            if run_in_background:
-                loglevel = LogLevel.get_str(Logger.CONSOLE_LEVEL)
-                command = ["janis", "--logLevel", loglevel, "resume", wid]
-                scriptdir = tm.get_path_for_component(
-                    tm.WorkflowManagerPath.configuration
+            if (
+                not run_in_background
+                and jc.template
+                and jc.template.template
+                and jc.template.template.can_run_in_foreground is False
+            ):
+                raise Exception(
+                    f"Your template '{jc.template.template.__class__.__name__}' is not allowed to run "
+                    f"in the foreground, try adding the '--background' argument"
                 )
-                logdir = tm.get_path_for_component(tm.WorkflowManagerPath.logs)
-                jc.template.template.submit_detatched_resume(
-                    wid=wid,
-                    command=command,
-                    scriptdir=scriptdir,
-                    logsdir=logdir,
-                    config=jc,
-                )
-
-                Logger.info("Submitted detatched engine")
-
-                if watch:
-                    Logger.log("Watching submitted workflow")
-                    tm.show_status_screen()
-            else:
-                if (
-                    jc.template
-                    and jc.template.template
-                    and jc.template.template.can_run_in_foreground is False
-                ):
-                    raise Exception(
-                        f"Your template '{jc.template.template.__class__.__name__}' is not allowed to run "
-                        f"in the foreground, try adding the '--background' argument"
-                    )
-                tm.resume()
-
+            tm.start_or_submit(run_in_background=run_in_background, watch=watch)
         else:
             tm.set_status(TaskStatus.DRY_RUN)
 
@@ -249,6 +231,37 @@ class WorkflowManager:
 
         return tm
 
+    def start_or_submit(self, run_in_background, watch=False):
+        # check container environment is loaded
+        metadb = self.database.workflowmetadata
+
+        jc = metadb.configuration
+        metadb.containertype = jc.container.__name__
+        metadb.containerversion = jc.container.test_available_by_getting_version()
+
+        # this happens for all workflows no matter what type
+        self.set_status(TaskStatus.QUEUED)
+
+        wid = metadb.wid
+
+        # resubmit the engine
+        if not run_in_background:
+            return self.resume()
+
+        loglevel = LogLevel.get_str(Logger.CONSOLE_LEVEL)
+        command = ["janis", "--logLevel", loglevel, "resume", "--foreground", wid]
+        scriptdir = self.get_path_for_component(self.WorkflowManagerPath.configuration)
+        logdir = self.get_path_for_component(self.WorkflowManagerPath.logs)
+        jc.template.template.submit_detatched_resume(
+            wid=wid, command=command, scriptdir=scriptdir, logsdir=logdir, config=jc
+        )
+
+        Logger.info("Submitted detatched engine")
+
+        if watch:
+            Logger.log("Watching submitted workflow")
+            self.show_status_screen()
+
     @staticmethod
     def from_path_with_wid(path, wid, readonly=False):
         """
@@ -263,7 +276,7 @@ class WorkflowManager:
         if not os.path.exists(path):
             raise FileNotFoundError(f"Couldn't find path '{path}'")
 
-        db = WorkflowDbManager.get_workflow_metadatadb(path, wid)
+        db = WorkflowDbManager.get_workflow_metadatadb(path, wid, readonly=readonly)
 
         if not wid:
             wid = db.wid  # .get_meta_info(InfoKeys.taskId)
@@ -287,7 +300,7 @@ class WorkflowManager:
 
         db.close()
 
-        tm = WorkflowManager(outdir=path, wid=wid, environment=env, readonly=True)
+        tm = WorkflowManager(outdir=path, wid=wid, environment=env, readonly=readonly)
         return tm
 
     @staticmethod
@@ -325,7 +338,7 @@ class WorkflowManager:
         #     )
         #     Logger.warn(txt)
 
-        if bl:
+        if bl is not None:
             self.poll_stored_metadata_with_blessed(bl)
         else:
             self.poll_stored_metadata_with_clear()
@@ -520,34 +533,42 @@ class WorkflowManager:
 
         is_allegedly_started = engine.test_connection()
 
-        if not is_allegedly_started:
+        if is_allegedly_started:
+            return
 
-            was_manually_started = False
+        if not isinstance(engine, Cromwell):
+            engine.start_engine()
+            return
 
-            # start mysql if it's cromwell, and then slightly change the config if one exists
-            if isinstance(engine, Cromwell):
-                if engine.config:
-                    if self.database.workflowmetadata.should_manage_database:
-                        # only here do we start mysql
-                        self.start_mysql_and_prepare_cromwell()
-                        self.database.workflowmetadata.manages_database = True
-                        was_manually_started = True
-                    else:
-                        Logger.info(
-                            "Skipping start mysql as database configuration already exists"
-                        )
+        additional_cromwell_params = []
+        if not engine.config:
+            Logger.info("Skipping start mysql as Janis is not managing the config")
+        else:
+            dbconfig: JanisDatabaseConfigurationHelper = self.database.workflowmetadata.dbconfig
+            dbtype = dbconfig.which_db_to_use()
+            if dbtype == dbconfig.DatabaseTypeToUse.existing:
+                engine.config.database = dbconfig.get_config_for_existing_config()
+            elif dbtype == dbconfig.DatabaseTypeToUse.filebased:
+                engine.config.database = dbconfig.get_config_for_filebased_db(
+                    path=self.get_path_for_component(self.WorkflowManagerPath.database)
+                    + "/cromwelldb"
+                )
+            elif dbtype == dbconfig.DatabaseTypeToUse.managed:
+                cromwelldb_config = self.start_mysql_and_prepare_cromwell_config()
+                additional_cromwell_params.append(
+                    "-Ddatabase.db.url=" + cromwelldb_config.db.url
+                )
+                engine.config.database = cromwelldb_config
+            else:
+                Logger.warn(
+                    "Skipping database config as '--no-database' option was provided."
+                )
 
-                else:
-                    Logger.info(
-                        "Skipping start mysql as Janis is not managing the config"
-                    )
-
-            if not was_manually_started:
-                engine.start_engine()
+            engine.start_engine(additional_cromwell_options=additional_cromwell_params)
             # Write the new engine details back into the database (for like PID, host and is_started)
             self.database.workflowmetadata.engine = engine
 
-    def start_mysql_and_prepare_cromwell(self):
+    def start_mysql_and_prepare_cromwell_config(self):
         scriptsdir = self.get_path_for_component(self.WorkflowManagerPath.mysql)
 
         containerdir = self.get_path_for_component(self.WorkflowManagerPath.database)
@@ -571,21 +592,14 @@ class WorkflowManager:
         )
         self.dbcontainer.start()
 
-        engine: Cromwell = self.get_engine()
-
-        engine.config.database = CromwellConfiguration.Database.mysql(username="root")
-        # engine.config.call_caching = CromwellConfiguration.CallCaching(enabled=True)
-
         port = self.dbcontainer.forwardedport
-        url = CromwellConfiguration.Database.MYSQL_URL.format(
-            url=f"127.0.0.1:{port}", database="cromwell"
+        return CromwellConfiguration.Database.mysql(
+            username="root", url=f"127.0.0.1:{port}"
         )
-        engine.start_engine(additional_cromwell_options=["-Ddatabase.db.url=" + url])
-        self.database.workflowmetadata.engine = engine
 
     def prepare_and_output_workflow_to_evaluate_if_required(
         self,
-        workflow,
+        tool: Tool,
         translator: TranslatorBase,
         validation: Optional[ValidationRequirements],
         batchrun: Optional[BatchRunRequirements],
@@ -595,19 +609,19 @@ class WorkflowManager:
         max_memory=None,
         allow_empty_container=False,
         check_files=True,
-    ):
+    ) -> Tool:
         if self.database.progressDB.has(ProgressKeys.saveWorkflow):
             return Logger.info(f"Saved workflow from task '{self.wid}', skipping.")
 
-        Logger.debug(f"Saving workflow with id '{workflow.id()}'")
+        Logger.debug(f"Saving workflow with id '{tool.id()}'")
 
         outdir_workflow = self.get_path_for_component(self.WorkflowManagerPath.workflow)
         translator.translate(
-            workflow,
+            tool,
             to_console=False,
             to_disk=True,
             hints=hints,
-            # This is just the base workflow, we're going to potentially transform the inputs
+            # This is just the base tool, we're going to potentially transform the inputs
             # and we only really care about the inputs for the workflow we're going to run.
             # We'll store the original workflow to run for provenance, but not to easily rerun
             write_inputs_file=False,
@@ -615,25 +629,26 @@ class WorkflowManager:
             allow_empty_container=allow_empty_container,
         )
 
-        Logger.info(f"Saved workflow with id '{workflow.id()}' to '{outdir_workflow}'")
+        Logger.info(f"Saved workflow with id '{tool.id()}' to '{outdir_workflow}'")
 
-        modifiers = [InputFileQualifierModifier]
+        modifiers = []
         if validation:
             modifiers.append(ValidatorPipelineModifier(validation))
 
         if batchrun:
             modifiers.append(BatchPipelineModifier(batchrun))
 
+        modifiers.append(InputFileQualifierModifier)
         # THIS ONE SHOULD BE LAST
 
         modifiers.append(InputChecker(check_file_existence=check_files))
 
-        workflow_to_evaluate, additional_inputs = PipelineModifierBase.apply_many(
-            modifiers, workflow, additional_inputs, hints=hints
+        tool_to_evaluate, additional_inputs = PipelineModifierBase.apply_many(
+            modifiers, tool, additional_inputs, hints=hints
         )
 
         translator.translate(
-            workflow_to_evaluate,
+            tool_to_evaluate,
             to_console=False,
             to_disk=True,
             with_resource_overrides=True,
@@ -648,31 +663,43 @@ class WorkflowManager:
         )
 
         self.evaluate_output_params(
-            wf=workflow_to_evaluate, additional_inputs=additional_inputs
+            wf=tool_to_evaluate, additional_inputs=additional_inputs
         )
 
         self.database.progressDB.set(ProgressKeys.saveWorkflow)
-        return workflow_to_evaluate
+        return tool_to_evaluate
+
+    def evaluate_output_params(self, wf: Tool, additional_inputs: dict):
 
-    def evaluate_output_params(self, wf: Workflow, additional_inputs: dict):
         mapped_inps = CwlTranslator().build_inputs_file(
             wf, recursive=False, additional_inputs=additional_inputs
         )
+        output_names: Dict[str, any] = {}
+        output_folders: Dict[str, any] = {}
+
+        if isinstance(wf, Workflow):
+            for o in wf.output_nodes.values():
+                output_names[o.id()] = self.evaluate_output_selector(
+                    o.output_name, mapped_inps
+                )
+                output_folders[o.id()] = self.evaluate_output_selector(
+                    o.output_folder, mapped_inps
+                )
 
         outputs: List[WorkflowOutputModel] = []
 
-        for o in wf.output_nodes.values():
+        for o in wf.tool_outputs():
             # We'll
             ext = None
-            innertype = o.datatype
-            iscopyable = isinstance(o.datatype, (File, Directory)) or (
-                isinstance(o.datatype, Array)
-                and isinstance(o.datatype.fundamental_type(), (File, Directory))
+            innertype = o.outtype
+            iscopyable = isinstance(o.outtype, (File, Directory)) or (
+                isinstance(o.outtype, Array)
+                and isinstance(o.outtype.fundamental_type(), (File, Directory))
             )
             while isinstance(innertype, Array):
                 innertype = innertype.subtype()
-            if isinstance(o.datatype, File):
-                ext = o.datatype.extension
+            if isinstance(o.outtype, File):
+                ext = o.outtype.extension
             outputs.append(
                 WorkflowOutputModel(
                     tag=o.id(),
@@ -680,13 +707,9 @@ class WorkflowManager:
                     original_path=None,
                     new_path=None,
                     timestamp=None,
-                    output_name=self.evaluate_output_selector(
-                        o.output_name, mapped_inps
-                    ),
-                    output_folder=self.evaluate_output_selector(
-                        o.output_folder, mapped_inps
-                    ),
-                    secondaries=o.datatype.secondary_files(),
+                    output_name=output_names.get(o.id()),
+                    output_folder=output_folders.get(o.id()),
+                    secondaries=o.outtype.secondary_files(),
                     extension=ext,
                 )
             )
@@ -1064,7 +1087,9 @@ class WorkflowManager:
     @staticmethod
     def mark_aborted(outputdir, wid) -> bool:
         try:
-            db = WorkflowDbManager.get_workflow_metadatadb(outputdir, wid)
+            db = WorkflowDbManager.get_workflow_metadatadb(
+                outputdir, wid, readonly=False
+            )
             db.please_abort = True
             db.kvdb.commit()
             Logger.info("Marked workflow as aborted, this may take some time full exit")
